#!/usr/bin/python
import copy
import os
import sepbase
import tempfile
import unittest
import re
import math

from batch_task_executor import *
import pbs_util


class ElasticBatchTaskComposer(BatchTaskComposer):
  def __init__(self, param_reader, njobs, nfiles_perjob):
    BatchTaskComposer.__init__(self)
    self.njobs = njobs
    self.nfiles_perjob = nfiles_perjob
    # Create all subjobs info
    self.subjobids = range(0, self.njobs)
    self.subjobfns_list = [None]*self.njobs
    path_out = param_reader.path_out
    # Create files for each subjob.
    for i in range(0,self.njobs):
       nfiles = self.nfiles_perjob
       self.subjobfns_list[i] = [None]*nfiles
       for j in range(0,nfiles):
         self.subjobfns_list[i][j] = '%s/%s-%s-%s.H' % (path_out,param_reader.prefix,i,j)
    return

  def GetSubjobScripts(self, subjobid):
    '''Return the scripts content for that subjob.'''
    subjobfns = self.subjobfns_list[subjobid]
    scripts=[]
    # Build scripts that create the files designated in subjobfns\
    jobname = subjobfns[0]
    shots=[]
    shots=(re.findall('\d+', jobname ))
    shot_num=int(shots[0])
    sub_shot=int(shots[1])
    args_ela=eq_args_from_cmdline
    #define shot position
    nshots=int(args_ela['nshots'])
    xstart=int(args_ela['x_source_in'])
    xend=int(args_ela['x_source_end'])
    if nshots == 1:
    	xsource=xstart
    else:
    	ninterv=float(nshots-1)
        xsource=int(xstart+shot_num*round((xend-xstart)/ninterv))
        if xsource > xend: xsource=xend
    cmd = '../Bin/elastic.x <%s mu=%s rho=%s par=%s Vx_rec=%s-%s-%s.H Vz_rec=%s-%s-%s.H sigmaxx_rec=%s-%s-%s.H sigmazz_rec=%s-%s-%s.H sigmaxz_rec=%s-%s-%s.H >/dev/null lame=%s dtflag=%s xsource=%s \n'  %(args_ela['lambda'],args_ela['mu'],args_ela['rho'],args_ela['par'],args_ela['Vx_rec'],shot_num,sub_shot,args_ela['Vz_rec'],shot_num,sub_shot,args_ela['sigmaxx_rec'],shot_num,sub_shot,args_ela['sigmazz_rec'],shot_num,sub_shot,args_ela['sigmaxz_rec'],shot_num,sub_shot,args_ela['lame'],args_ela['dtflag'],xsource) 
    scripts.append(cmd)
    cmd = 'Math file1=%s-%s-%s.H file2=%s-%s-%s.H exp=\'(file1+file2)*0.5\' >%s ' %(args_ela['sigmaxx_rec'],shot_num,sub_shot,args_ela['sigmazz_rec'],shot_num,sub_shot,jobname)
    scripts.append(cmd)
    return scripts, str(subjobid)

  def GetSubjobsInfo(self):
    '''Should return two lists, subjobids and subjobfns.'''
    return copy.deepcopy(self.subjobids), copy.deepcopy(self.subjobfns_list)


if __name__ == '__main__':
  print "Run BatchTaskExecutor_elastic_foward with params:", sys.argv
  # Check basic environment variable setup.
  assert 'SEP' in os.environ, '!Environment var SEP is not set yet, should have your seplib enviroment set up first! Check paths like /opt/SEP/'
  assert 'RSF' in os.environ, '!Environment var RSF is not set yet, should have your RSF environment set up first! Check paths like /opt/RSF'
  eq_args_from_cmdline,args = sepbase.parse_args(sys.argv)
  #dict_args = sepbase.RetrieveAllEqArgs(eq_args_from_cmdline)
  param_reader = pbs_util.JobParamReader(eq_args_from_cmdline)
  njobs=int(eq_args_from_cmdline['nshots'])
  nfiles_perjob=1
  elastic_btc = ElasticBatchTaskComposer(param_reader,njobs,nfiles_perjob)
  bte = BatchTaskExecutor(param_reader)
  prefix = param_reader.prefix
  bte.LaunchBatchTask(prefix, elastic_btc)

  print 'Run Successful!'
  
