#!/usr/bin/python
import copy
import os
import sepbase
import tempfile
import unittest
import re
import math

from batch_task_executor import *
import pbs_util


class ElasticBatchTaskComposer(BatchTaskComposer):
  def __init__(self, param_reader, njobs, nfiles_perjob):
    BatchTaskComposer.__init__(self)
    self.njobs = njobs
    self.nfiles_perjob = nfiles_perjob
    # Create all subjobs info
    self.subjobids = range(0, self.njobs)
    self.subjobfns_list = [None]*self.njobs
    path_out = param_reader.path_out
    # Create files for each subjob.
    for i in range(0,self.njobs):
    	nfiles = self.nfiles_perjob
    	self.subjobfns_list[i] = [None]*nfiles
    	for j in range(0,nfiles):
			self.subjobfns_list[i][j] = '%s/%s-lambda-%s.H' % (path_out,param_reader.prefix,i)
    return

  def GetSubjobScripts(self, subjobid):
    '''Return the scripts content for that subjob.'''
    subjobfns = self.subjobfns_list[subjobid]
    scripts=[]
    # Build scripts that create the files designated in subjobfns\
    jobname = subjobfns[0]
    shots=[]
    shots=(re.findall('\d+', jobname ))
    shot_num=int(shots[0])
    args_ela=eq_args_from_cmdline
    #define shot position file
    cmd = 'touch Par/%s-%s.p \n' %(param_reader.prefix,shot_num)
    scripts.append(cmd)
    cmd = 'echo 1 >> Par/%s-%s.p \n' %(param_reader.prefix,shot_num)
    scripts.append(cmd)
    cmd = 'awk \'NR==%s\' %s >> Par/%s-%s.p \n' %(shot_num+4,args_ela['par_sou'],param_reader.prefix,shot_num)
    scripts.append(cmd)
    #Windowing data file
    #cmd = 'Window <%s n3=1 f3=%s | Pad beg1=200 | Bandpass flo=2 fhi=45 > data-%s-%s.H \n' %(args_ela['data'],shot_num,param_reader.prefix,shot_num)
    cmd = 'Window <%s n3=1 f3=%s | Bandpass flo=2 fhi=45 > data-%s-%s.H \n' %(args_ela['data'],shot_num,param_reader.prefix,shot_num)
    scripts.append(cmd)
    #Running the executable   >deltalambda.H
    cmd = '../Bin/Bornfast_pressure_elastic.x < %s rho=%s mu=%s deltadata=data-%s-%s.H wavelet=%s sources=Par/%s-%s.p recs=%s deltarho=%s-rho-%s.H deltamu=%s-mu-%s.H >%s-lambda-%s.H \n' %(args_ela['lambda'],args_ela['rho'],args_ela['mu'],param_reader.prefix,shot_num,args_ela['wavelet'],param_reader.prefix,shot_num,args_ela['par_rec'],param_reader.prefix,shot_num,param_reader.prefix,shot_num,param_reader.prefix,shot_num)
    scripts.append(cmd)
    return scripts, str(subjobid)

  def GetSubjobsInfo(self):
    '''Should return two lists, subjobids and subjobfns.'''
    return copy.deepcopy(self.subjobids), copy.deepcopy(self.subjobfns_list)


if __name__ == '__main__':
  print "Run BatchTaskExecutor_elastic_imaging with params:", sys.argv
  # Check basic environment variable setup.
  assert 'SEP' in os.environ, '!Environment var SEP is not set yet, should have your seplib enviroment set up first! Check paths like /opt/SEP/'
  assert 'RSF' in os.environ, '!Environment var RSF is not set yet, should have your RSF environment set up first! Check paths like /opt/RSF'
  eq_args_from_cmdline,args = sepbase.parse_args(sys.argv)
  #dict_args = sepbase.RetrieveAllEqArgs(eq_args_from_cmdline)
  param_reader = pbs_util.JobParamReader(eq_args_from_cmdline)
  njobs=int(eq_args_from_cmdline['nshots'])
  nfiles_perjob=1
  elastic_btc = ElasticBatchTaskComposer(param_reader,njobs,nfiles_perjob)
  bte = BatchTaskExecutor(param_reader)
  prefix = param_reader.prefix
  bte.LaunchBatchTask(prefix, elastic_btc)


  print 'Run Successful!'
  
