#!/usr/bin/python
import copy
import os
import sepbase
import tempfile
import unittest
import re
import math

from batch_task_executor import *
import pbs_util


class ElasticBatchTaskComposer(BatchTaskComposer):
  def __init__(self, param_reader, njobs, nfiles_perjob):
    BatchTaskComposer.__init__(self)
    self.njobs = njobs
    self.nfiles_perjob = nfiles_perjob
    # Create all subjobs info
    self.subjobids = range(0, self.njobs)
    self.subjobfns_list = [None]*self.njobs
    path_out = param_reader.path_out
    # Create files for each subjob.
    for i in range(0,self.njobs):
       nfiles = self.nfiles_perjob
       self.subjobfns_list[i] = [None]*nfiles
       for j in range(0,nfiles):
         self.subjobfns_list[i][j] = '%s/%s-%s-%s.H' % (path_out,param_reader.prefix,i,j)
    return

  def GetSubjobScripts(self, subjobid):
    '''Return the scripts content for that subjob.'''
    subjobfns = self.subjobfns_list[subjobid]
    scripts=[]
    # Build scripts that create the files designated in subjobfns\
    jobname = subjobfns[0]
    shots=[]
    shots=(re.findall('\d+', jobname ))
    shot_num=int(shots[0])
    sub_shot=int(shots[1])
    args_ela=eq_args_from_cmdline
    #define shot position
    nshots=int(args_ela['nshots'])
    xstart=int(args_ela['x_source_in'])
    xend=int(args_ela['x_source_end'])
    if nshots == 1:
    	xsource=xstart
    else:
    	ninterv=float(nshots-1)
        xsource=int(xstart+shot_num*round((xend-xstart)/ninterv))
        if xsource > xend: xsource=xend
    #creating the data to be backpropagated
    cmd = 'Window <%s n3=1 f3=%s | Transp | Pad beg1=%s end1=%s | Add scale=0.5 >pressure-%s.H \n'%(args_ela['data'],shot_num,args_ela['pad'],args_ela['pad'],shot_num)
    scripts.append(cmd)
    cmd = '../Bin/rtm.x <%s mu=%s rho=%s Vx_rec=null.H Vz_rec=null.H sigmaxz_rec=null.H par=%s bkg_data=0 nodes=16 lame=1 adj=1 sigmaxx_rec=pressure-%s.H sigmazz_rec=pressure-%s.H dmu=dmu-%s.H drho=drho-%s.H  dlambda=%s >/dev/null xsource=%s \n' %(args_ela['lambda'],args_ela['mu'],args_ela['rho'],args_ela['par'],shot_num,shot_num,shot_num,shot_num,jobname,xsource)
    scripts.append(cmd)
    return scripts, str(subjobid)

  def GetSubjobsInfo(self):
    '''Should return two lists, subjobids and subjobfns.'''
    return copy.deepcopy(self.subjobids), copy.deepcopy(self.subjobfns_list)


if __name__ == '__main__':
  print "Run BatchTaskExecutor_elastic_rtm with params:", sys.argv
  # Check basic environment variable setup.
  assert 'SEP' in os.environ, '!Environment var SEP is not set yet, should have your seplib enviroment set up first! Check paths like /opt/SEP/'
  assert 'RSF' in os.environ, '!Environment var RSF is not set yet, should have your RSF environment set up first! Check paths like /opt/RSF'
  eq_args_from_cmdline,args = sepbase.parse_args(sys.argv)
  #dict_args = sepbase.RetrieveAllEqArgs(eq_args_from_cmdline)
  param_reader = pbs_util.JobParamReader(eq_args_from_cmdline)
  njobs=int(eq_args_from_cmdline['nshots'])
  nfiles_perjob=1
  elastic_btc = ElasticBatchTaskComposer(param_reader,njobs,nfiles_perjob)
  bte = BatchTaskExecutor(param_reader)
  prefix = param_reader.prefix
  bte.LaunchBatchTask(prefix, elastic_btc)

  print 'Run Successful!'
  
